@@ -1,5 +1,6 @@
 from webob import Request
 from webtest import TestApp
+from nose.tools import eq_, ok_, assert_raises
 
 def simpleapp(environ, start_response):
     status = '200 OK'
@@ -12,7 +13,7 @@ def simpleapp(environ, start_response):
         'The get is %r' % request.GET,
         ' and Val is %s\n' % request.GET.get('name'),
         'The languages are: %s\n' % request.accept_language.best_matches('en-US'),
-        'The accepttypes is: %s\n' % request.accept.best_match(['text/html', 'application/xml']),
+        'The accepttypes is: %s\n' % request.accept.best_match(['application/xml', 'text/html']),
         'post is %r\n' % request.POST,
         'params is %r\n' % request.params,
         'cookies is %r\n' % request.cookies,
@@ -33,18 +34,18 @@ def test_gets():
     res = app.get('/')
     print res
     assert 'Hello' in res
-    assert "get is MultiDict([])" in res
-    assert "post is <NoVars: Not a POST request>" in res
-    
+    assert "get is GET([])" in res
+    assert "post is <NoVars: Not a form request>" in res
+
     res = app.get('/?name=george')
-    res.mustcontain("get is MultiDict([('name', 'george')])")
+    res.mustcontain("get is GET([('name', 'george')])")
     res.mustcontain("Val is george")
 
 def test_language_parsing():
     app = TestApp(simpleapp)
     res = app.get('/')
     assert "The languages are: ['en-US']" in res
-    
+
     res = app.get('/', headers={'Accept-Language':'da, en-gb;q=0.8, en;q=0.7'})
     assert "languages are: ['da', 'en-gb', 'en', 'en-US']" in res
 
@@ -55,12 +56,69 @@ def test_mime_parsing():
     app = TestApp(simpleapp)
     res = app.get('/', headers={'Accept':'text/html'})
     assert "accepttypes is: text/html" in res
-    
+
     res = app.get('/', headers={'Accept':'application/xml'})
     assert "accepttypes is: application/xml" in res
-    
+
     res = app.get('/', headers={'Accept':'application/xml,*/*'})
-    assert "accepttypes is: application/xml" in res
+    assert "accepttypes is: application/xml" in res, res
+
+
+def test_accept_best_match():
+    assert not Request.blank('/').accept
+    assert not Request.blank('/', headers={'Accept': ''}).accept
+    req = Request.blank('/', headers={'Accept':'text/plain'})
+    ok_(req.accept)
+    assert_raises(ValueError, req.accept.best_match, ['*/*'])
+    req = Request.blank('/', accept=['*/*','text/*'])
+    eq_(req.accept.best_match(['application/x-foo', 'text/plain']), 'text/plain')
+    eq_(req.accept.best_match(['text/plain', 'application/x-foo']), 'text/plain')
+    req = Request.blank('/', accept=['text/plain', 'message/*'])
+    eq_(req.accept.best_match(['message/x-foo', 'text/plain']), 'text/plain')
+    eq_(req.accept.best_match(['text/plain', 'message/x-foo']), 'text/plain')
+
+def test_from_mimeparse():
+    # http://mimeparse.googlecode.com/svn/trunk/mimeparse.py
+    supported = ['application/xbel+xml', 'application/xml']
+    tests = [('application/xbel+xml', 'application/xbel+xml'),
+             ('application/xbel+xml; q=1', 'application/xbel+xml'),
+             ('application/xml; q=1', 'application/xml'),
+             ('application/*; q=1', 'application/xbel+xml'),
+             ('*/*', 'application/xbel+xml')]
+
+    for accept, get in tests:
+        req = Request.blank('/', headers={'Accept':accept})
+        assert req.accept.best_match(supported) == get, (
+            '%r generated %r instead of %r for %r' % (accept, req.accept.best_match(supported), get, supported))
+
+    supported = ['application/xbel+xml', 'text/xml']
+    tests = [('text/*;q=0.5,*/*; q=0.1', 'text/xml'),
+             ('text/html,application/atom+xml; q=0.9', None)]
+
+    for accept, get in tests:
+        req = Request.blank('/', headers={'Accept':accept})
+        assert req.accept.best_match(supported) == get, (
+            'Got %r instead of %r for %r' % (req.accept.best_match(supported), get, supported))
+
+    supported = ['application/json', 'text/html']
+    tests = [('application/json, text/javascript, */*', 'application/json'),
+             ('application/json, text/html;q=0.9', 'application/json')]
+
+    for accept, get in tests:
+        req = Request.blank('/', headers={'Accept':accept})
+        assert req.accept.best_match(supported) == get, (
+            '%r generated %r instead of %r for %r' % (accept, req.accept.best_match(supported), get, supported))
+
+    offered = ['image/png', 'application/xml']
+    tests = [
+        ('image/png', 'image/png'),
+        ('image/*', 'image/png'),
+        ('image/*, application/xml', 'application/xml'),
+    ]
+
+    for accept, get in tests:
+        req = Request.blank('/', accept=accept)
+        eq_(req.accept.best_match(offered), get)
 
 def test_headers():
     app = TestApp(simpleapp)
@@ -80,7 +138,7 @@ def test_headers():
         "params is NestedMultiDict([('foo', 'bar'), ('baz', '')])",
         "if_none_match: <ETag etag001 or etag002>",
         )
-    
+
 def test_bad_cookie():
     req = Request.blank('/')
     req.headers['Cookie'] = '070-it-:><?0'
@@ -138,5 +196,5 @@ def test_copy():
     old_body_file = req.body_file
     req.make_body_seekable()
     assert req.body_file is old_body_file
-    
-    
+
+
